# Convert_NetCDF4_Variable_To_Raster.py
#
# Converts a single variable in a netCDF4 file to a GIS-compatible
# raster in .img (ERDAS Imagine) format. Requires MGET and ArcGIS.
# Takes two parameters: the path to the netCDF4 file and the name of
# the variable within it, e.g.:
#
# Convert_NetCDF4_Variable_To_Raster.py C:\OceanColor\A20021822014212.L3m_MC_CHL_chlor_a_4km.nc chlor_a
#
# The netCDF variable must have complete CF-compatible metadata to
# determine how it is georeferenced. At this time, only WGS 1984
# equiangular (a.k.a. "geographic") coordinate systems are supported.
# If the netCDF uses another coordinate system, it can be specified
# manually by tweaking the code below.
#
# The raster will be created in the same directory with the same
# filename, but with the variable name appended with an underscore.
# For the example above, the raster will be:
#
# C:\OceanColor\A20021822014212.L3m_MC_CHL_chlor_a_4km_chlor_a.img

# Extract command line arguments.

import sys
ncFile = sys.argv[1]
ncVariable = sys.argv[2]

# Initialize MGET's logging system. This is optional. If not done,
# MGET will not log any messages.

from GeoEco.Logging import Logger
Logger.Initialize()

# Initialize MGET's instance of the ArcGIS geoprocessor. This can take
# several seconds and is probably the slowest part of this script.

from GeoEco.ArcGIS import GeoprocessorManager
GeoprocessorManager.InitializeGeoprocessor()

# Open the netCDF file with MGET.

from GeoEco.Datasets.NetCDF import NetCDFFile
nc = NetCDFFile(ncFile)

# Convert the netCDF variable to a raster. This is done by first
# telling MGET about the ArcGIS workspace and how to name rasters
# within it, and then by asking MGET to import datasets into the
# workspace. This code can be modified to convert multiple datasets at
# once (e.g. convert a directory tree of netCDFs to a directory tree
# of rasters).

import os
from GeoEco.Datasets.ArcGIS import ArcGISWorkspace, ArcGISRaster

ws = ArcGISWorkspace(os.path.dirname(ncFile),
                     ArcGISRaster,
                     pathCreationExpressions=[os.path.splitext(os.path.basename(ncFile))[0] + '_' + ncVariable + '.img'])

ws.ImportDatasets(nc.QueryDatasets("VariableName = '%s'" % ncVariable),
                  mode='Replace',                                   # Overwrite existing raster.
                  calculateStatistics=True,
                  buildPyramids=True)
